library unify.impl.browser;

import 'dart:async';
import 'dart:convert';
import 'dart:html';

import 'package:meta/meta.dart';

import '../../unify.dart';
import 'common/http.dart';

class UnifyClientImpl extends UnifyHttpClient {
  UnifyClientImpl(String email, String password, UnifyAccessToken token,
      UnifyApiEndpoints endpoints)
      : super(email, password, token, endpoints);

  @override
  Future<dynamic> sendHttpRequest(
    String url, {
    bool needsToken: true,
    String extract,
    dynamic body,
    String method,
    Uri baseUrl,
  }) async {
    var uri = baseUrl ?? endpoints.userApiUrl.resolve(url);

    var request = HttpRequest();
    request.open(method ?? body == null ? "GET" : "POST", uri.toString());
    if (needsToken) {
      if (!isCurrentTokenValid(true)) {
        await login();
      }
      request.setRequestHeader("Authorization", "Bearer ${token.accessToken}");
    }

    if (body != null) {
      request.setRequestHeader(
          "Content-Type", "application/json; charset=utf-8");
      request.send(const JsonEncoder().convert(body));
    } else {
      request.send();
    }

    await request.onLoadEnd.first;

    var content = request.responseText;
    if (request.status != 200) {
      throw Exception(
          "Failed to perform action. (Status Code: ${request.status})\n${content}");
    }
    var result = const JsonDecoder().convert(content);

    if (result is! Map) {
      if (extract != null) {
        return result[extract];
      }
    }

    return result;
  }

  @override
  Future<dynamic> postHttp(
    String url, {
    String extract,
    Map<String, dynamic> body,
    String contentType,
    Uri baseUrl,
    bool raw = false,
  }) async {
    var uri = baseUrl ?? endpoints.userApiUrl.resolve(url);

    var request = HttpRequest();
    request.open(body == null ? "GET" : "POST", uri.toString());

    request.setRequestHeader(
        "Content-Type", contentType ?? "application/x-www-form-urlencoded");
    request.send(body);

    await request.onLoadEnd.first;

    var content = request.responseText;
    if (request.status != 200) {
      throw Exception(
          "Failed to perform action. (Status Code: ${request.status})\n${content}");
    }

    if (raw) return content;

    var result = const JsonDecoder().convert(content);

    if (result is! Map) {
      if (extract != null) {
        return result[extract];
      }
    }

    return result;
  }

  @override
  Future close() async {}

  @override
  Future<Map<String, dynamic>> uploadFile(
    String url, {
    @required List<Object> fileBits,
    @required String name,
    @required String path,
  }) async {
    File _data = File(fileBits, name);
    final request = HttpRequest();
    request.open('POST', url);
    request.upload.onProgress.listen((ProgressEvent e) {
      var progress = (e.loaded * 100 ~/ e.total).toString() + '%';
      print("Progress: $progress");
    });
    request.onLoad.listen((e) {
      print('Uploaded!');
    });

    await request.send(_data);

    var content = request.responseText;
    if (request.status != 200) {
      throw Exception(
          "Failed to perform action. (Status Code: ${request.status})\n${content}");
    }

    var result = const JsonDecoder().convert(content);

    return result;
  }
}
